<?php
/* $Id: class_album.inc,v 1.26 2006/11/09 00:15:29 vhmauery Exp $ */

/*
Acidfree Photo Albums for Drupal
Copyright (C) 2005 Vernon Mauery

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/

/**
 * @file
 * The is the class implementation of the album.  All the acidfree elements
 * must be defined in this same manner, which makes creating a new class
 * relatively simple -- copy one of the existing classes and modify the 
 * relevant functions.
 */

function class_album_register() {
    $class = new stdClass();
    $class->class = 'album';
    /* i.e. reports image/pjpeg as mime type for jpegs.  grrr. */
    $class->mime_ext = Array();
    $class->form_alter = '_class_album_form_alter';
    $class->nodeapi = '_class_album_nodeapi';
    $class->access = 'create acidfree albums';
    $class->add_form = '_class_album_add_form';
    $class->edit_form = '_class_album_edit_form';
    return $class;
}

function _album_create_links(&$term) {
    global $acidfree_types;
    if (isset($term)) {
        $tpath = "/{$term->tid}";
    } else {
        $tpath = "";
    }
    $links = Array();
    $show_box = node_access('update', $term) ||
        (_acidfree_can_create() && $term->share) ||
        user_access('can upload to any album');
    if (!$show_box)
        return $links;
    foreach ($acidfree_types as $type) {
        if (user_access($type->access)){
            $type->path = ($type->class == 'album') ? 'acidfree' : $type->class;
            if(preg_match('/^[aeiou]/',$type->class)){ 
                $links[] = l(t('Add an !type', array('!type' => $type->class)),
                        "node/add/{$type->path}{$tpath}");
            }
            else
            {
                $links[] = l(t('Add a !type', array('!type' => $type->class)),
                        "node/add/{$type->path}{$tpath}");            
            }
        }
    }
    if (user_access('acidfree mass import'))
        $links[] = l(t('Mass import'), "node/add/acidfree/mass{$tpath}");
    return theme('item_list',$links);
}

function _acidfree_get_album_thumb(&$node) {
    $child = _acidfree_get_children($node->tid, 1, 0, false, 'RAND()');

    if (!$child) {
        $child = _acidfree_get_children($node->tid, 1, 0, true, 'RAND()');
        if (!$child) {
            return '';
        }
        return _acidfree_get_album_thumb(acidfree_get_node_by_id($child[0]));
    } else {
        $thumb = acidfree_get_node_by_id($child[0]);
        $thumb = _acidfree_get_thumb_url($thumb);
    }   
    return $thumb;
}

function acidfree_album_contents_submit($form_id, $form) {
    $op = isset($_POST['op']) ? $_POST['op'] : '';
    //dump($form, true);
    $node = $form['album'];
    global $acidfree_types;
    $parent = $form['parent'];
    $targets = array();
    foreach ($form['nodes'] as $fnode) {
        $n = acidfree_get_node_by_id($fnode['nid']);
        // troublesome keys -- they use an array on the form
        // and then don't know how to handle it when it is passed back
        $trouble_keys = array('path','og_nodeapi');
        // this takes care of class extras and title
        foreach ($fnode as $k => $v) {
            if (is_array($v) && in_array(strtolower($k), $trouble_keys)) {
                foreach ($v as $vk => $vv) {
                    $n->$vk = $vv;
                }
            } else
                $n->$k = $v;
        }
        if ($fnode['checked'] == '1') {
            // move, copy, delete the node
            switch ($op) {
            case t('Delete'):
                if ($n->class != 'album' && count($n->parent) > 1) {
                    // delete from this album only
                    $key = array_search($node, $n->parent);
                    unset($n->parent[$key]);
                    drupal_set_message(t("removed '%item' from album",
                            array('%item' => $n->title)));
                } else {
                    node_delete($n->nid);
                }
                continue;
            case t('Copy'):
                if ($n->class != 'album') {
                    // add dest->nid to parent list
                    $n->parent[] = $parent;
                } else {
                    _acidfree_clone_album($n, $parent);
                }
                break;
            case t('Move'):
                // replace node->nid with dest->nid in parent list
                if ($n->class != 'album') {
                    $key = array_search($node, $n->parent);
                    unset($n->parent[$key]);
                    $n->parent[$parent] = $parent;
                } else {
                    $parents = _acidfree_get_ancestors(acidfree_get_node_by_id($parent));
                    $awry = ($parent == $n->nid);
                    foreach ($parents as $p) {
                        if ($p->nid == $n->nid) {
                            $awry = true;
                            break;
                        }
                    }
                    if (!$awry) {
                        $n->parent = array($parent);
                    } else {
                        drupal_set_message(t("Could not set '%item' to be own ancestor album",
                                array('%item' => $n->title)), 'error');
                    }
                }
                break;
            }
        }
        node_save($n);
    }
    // not sure why, but even though we pass the nodes around as references,
    // somewhere things go awry and acidfree_get_node_by_id returns the 
    // old copy that is not the one we saved.  so we just reload.
    drupal_set_message(t('Changes saved.'));
    $pager = _acidfree_make_pager_query();
    drupal_goto("album/{$form['album']}/contents", $pager);
}

function theme_acidfree_album_contents($form) {
    if (count($form['nodes']) == 0)
        return "This album is empty<br>";
    $output .= drupal_render($form['pager']);
    $output .= '<div class="container-inline" style="padding: 5px;">';
    foreach (element_children($form['actionstop']) as $key) {
        $output .= drupal_render($form['actionstop'][$key]) . '&nbsp;&nbsp;';
    }
    $output .= '</div>';

    foreach (element_children($form['nodes']) as $child) {
        $row[] = Array('data' => drupal_render($form['nodes'][$child]['checked']));
        $node = $form['nodes'][$child]['#node'];
        $cell = theme("acidfree_print_thumb_{$node->type}", $node);
        foreach (array_keys(acidfree_form_elements($form['nodes'][$child])) as $key) {
            $cell .= drupal_render($form['nodes'][$child][$key]);
        }
        $row[] = Array( 'data' => $cell,
                'class' => 'album-cell');
        unset($cell);
        $cell .= drupal_render($form['nodes'][$child]['title']);
        $cell .= drupal_render($form['nodes'][$child]['term']['name']);
        $cell .= drupal_render($form['nodes'][$child]['body']);
        $cell .= drupal_render($form['nodes'][$child]['term']['description']);
        $cell .= drupal_render($form['nodes'][$child]['parent']);
        $row[] = Array('data' => $cell, 'class' => 'album-cell');
        $cell = drupal_render($form['nodes'][$child]);
        $row[] = Array('data' => $cell, 'class' => 'album-cell');
        $rows[] = Array('data' => $row);
        unset($row);
    }
    $output .= theme('table', NULL, $rows);
    $output .= '<div class="container-inline" style="padding: 5px;">';
    foreach (element_children($form['actionsbot']) as $key) {
        $output .= drupal_render($form['actionsbot'][$key]) . '&nbsp;&nbsp;';
    }
    $output .= '</div>';
//    $output .= '<pre>'; $output .= dump($form); $output .= '</pre>';
    return $output.drupal_render($form);
}

function acidfree_album_contents(&$term) {
    $form = array();
    drupal_set_title(t('Album Contents'));
    drupal_set_breadcrumb($bc = acidfree_make_breadcrumbs($term));
    $ncols = variable_get('acidfree_cols', 5);
    $nrows = variable_get('acidfree_rows', 3);
    if ($nrows == -1) {
        $nrows = 5;
    }
    $nodes_per_page = $ncols*$nrows;
    $nodes = _acidfree_get_children($term, $nodes_per_page, ALBUM_PAGER, true);
    $term = taxonomy_get_term($term);
    if (count($nodes) == 0)
        return "This album is empty<br>";
    $form['album'] = array(
            '#type' => 'hidden',
            '#value' => $term->tid,
        );
    $form['pager'] = array(
            '#value' => theme('pager', NULL, $nodes_per_page, ALBUM_PAGER),
        );
    $form['actionstop']['select'] = array(
            '#type' => 'select',
            '#default_value' => '0',
            '#options' => array('0'=>' -- ', 'all'=>t('All'), 'none'=>t('None'), 'invert'=>t('Invert')),
            '#attributes' => array('onchange' => 'select_nodes(this);'),
            '#prefix' => '<div class="form-item"><label for="edit-select">Select: </label></div>',
        );
    $form['actionstop']['parent'] = _acidfree_parent_select($term);
    //$form['actionstop']['parent'] = _acidfree_parent_select(null, $node->nid, null, false, array(), array('onchange' => 'update_parent_selects(this);'));
    $form['actionstop']['parent']['#prefix'] = '<div class="form-item"><label for="edit-parent">Destination: </label></div>';
    $form['actionstop']['move'] = array(
            '#type' => 'submit',
            '#value' =>t('Move'),
        );
    $form['actionstop']['copy'] = array(
            '#type' => 'submit',
            '#value' =>t('Copy'),
        );
    $form['actionstop']['delete'] = array(
            '#type' => 'submit',
            '#value' =>t('Delete'),
        );
    $form['actionstop']['submit'] = array(
            '#type' => 'submit',
            '#value' =>t('Submit'),
        );

    $form['nodes']['#tree'] = true;
    $i = 0;
    $form_alter_mods = module_implements('form_alter');
    foreach ($nodes as $child) {
        $child = acidfree_get_node_by_id($child);
        $form['nodes'][$i] = node_form($child, $param);
        unset($form['nodes'][$i]['submit']);
        unset($form['nodes'][$i]['preview']);
        unset($form['nodes'][$i]['delete']);
        unset($form['nodes'][$i]['#after_build']);
        $form['nodes'][$i]['type'] = array('#value' => $child->type);
        $form['nodes'][$i]['checked'] = array(
                '#type' => 'checkbox',
                '#default_value' => 0,
                '#tree' => true,
            );
        $form['nodes'][$i]['nid'] = array(
                '#type' => 'hidden',
                '#value' => $child->nid,
                '#tree' => true,
            );
        $form['nodes'][$i]['#tree'] = true;
        $form['nodes'][$i]['#node'] = $child;
        foreach ($form_alter_mods as $module) {
            $function = $module .'_form_alter';
            $form_id = $child->type.'_node_form';
            $function($form_id, $form['nodes'][$i]);
        }
        //dump_msg($form['nodes'][$i]);
        unset($form['nodes'][$i]['type']);
        $i++;
        continue;
    }
    $form['actionsbot'] = $form['actionstop'];
    return $form;
}

/**
 * This views generates the customized acidfree views based on the standard view
 *
 * @param $view_name
 *   The name or id of the view to load
 * @param $args
 *   The arguments from the end of the url. Usually passed from the menu system.
 *
 * @return
 *   The view page.
 */
function acidfree_album_view() {
    $args = func_get_args();
    $view_name = array_shift($args);
    $view = views_get_view($view_name);

    $album = _album_from_tid($args[0]);
    if (!$album->order_by) {
        $album->order_by = "node.nid DESC";
    }
    $album_sort = explode(' ', $album->order_by);
    $view->sort[1] = array(
            'vid' => $view->vid,
            'position' => 1,
            'field' => $album_sort[0],
            'sortorder' => $album_sort[1],
            'options' => NULL,
            'tablename' => NULL,
            'id' => $album_sort[0],
        );

    if (!$view) {
        drupal_not_found();
        exit;
    }
    $view->debug = true;

    $output = views_build_view('page', $view, $args, $view->use_pager, $view->nodes_per_page);
    if ($output === FALSE) {
        drupal_not_found();
        exit;
    }

    return $output;
}

function theme_views_view_album_grid_view($view, $type, $nodes, $teasers = false, $links = true) {
    $term = taxonomy_get_term($view->args[0]);
    drupal_set_breadcrumb(acidfree_make_breadcrumbs($term));
    drupal_set_title($term->name);
    $ncols = variable_get('acidfree_cols', 5);
    if (count($nodes) == 0) 
        return "This album is empty<br>";
    $output = theme('pager', NULL, 1, ALBUM_PAGER);
    foreach ($nodes as $node) {
        $node = node_load(array('nid' => $node->nid));
        $func = "acidfree_print_thumb_{$node->type}";
        $row[] = Array(
                'data' => theme("acidfree_print_thumb_{$node->type}",
                                $node),
                'class' => 'album-cell');

        if (++$count % $ncols == 0) {
            $rows[] = Array('data' => $row, 'class' => 'none');
            unset($row);
        }
    }
    if (isset($row))
        $rows[] = Array('data' => $row, "class" => "none");
    $output .= theme('table', NULL, $rows, Array('id' => 'acidfree-main'), false);
    return $output;
}

function theme_views_view_album_list_view($view, $type, $nodes, $teasers = false, $links = true) {
    $term = taxonomy_get_term($view->args[0]);
    drupal_set_breadcrumb(acidfree_make_breadcrumbs($term));
    drupal_set_title($term->name);
    $ncols = variable_get('acidfree_cols', 5);
    $output .= theme('pager', NULL, 1, ALBUM_PAGER);
    if (count($nodes) == 0) 
        return "This album is empty<br>";
    $output .= "<ol class='album-list'>\n";
    $i = _acidfree_get_pager_offset_from_path(ALBUM_PAGER) * $nodes_per_page;
    foreach ($nodes as $node) {
        $node = node_load(array('nid' => $node->nid));
        //drupal_set_message("{$node->title}, nid {$node->nid}, type {$node->type}");
        $item = "<div class='list-number'>".(++$i)."</div>";
        $float_class = "acidfree-list-".($i & 0x1 ? "odd" : "even");
        $item .= "<div class='$float_class'>";
        $image = theme('image', _acidfree_get_thumb_url($node),
                            $node->title, $node->title,
                            array('class' => "acidfree-plain acidfree-list",
                                'id' => "node-{$node->nid}"), false);
        $item .= l($image, "node/{$node->nid}",
                        NULL, NULL, NULL, FALSE, TRUE);
        $item .= "<div class='image-caption'>{$node->title}</div>";
        $item .= "</div>";
        $item .= "<div class='main acidfree-body'>".check_markup($node->body)."</div>";
        $rows[] = $item;
    }
    $output .= '<div class="acidfree-list-row">';
    $output .= implode('</div>'."\n".'<div class="acidfree-list-row">', $rows);
    $output .= "</div>\n";
    return $output;
}

function theme_acidfree_print_thumb_acidfree(&$node) {
    $album_link = "album/{$node->tid}";
    $image = _acidfree_get_album_thumb($node);
    if ($image != '') {
        $image = l(theme('image', $image, $node->title, $node->title, 
                    array(
                        'class'=>'album-thumb',
                        'id'=>"acidfree-thumb{$node->nid}"),
                    false),
                $album_link, array(), NULL, NULL, true, true);
    }
    $size = variable_get('acidfree_thumb_dim', IMAGE_THUMB_SIZE) + 40;
    $output .= "<div class='fullslide' style='width: {$size}px;'>\n";
    $output .= "<div class='folder' onclick='document.location=\"".url($album_link, null, null, true)."\"'>";
    // XXX: this empty span is to make IE render vertical alignment correctly
    $output .= "<span></span>\n";
    $output .= $image;
    $output .= '</div><div>'.$node->title;
    //$output .= '</div><div>'.l($node->title, $album_link);
    $output .= "</div></div>\n";
    return $output;
}

function _album_from_tid($tid) {
    $nid = db_result(db_query("SELECT aid FROM {acidfree_album} WHERE tid=%d", $tid));
    return acidfree_get_node_by_id($nid);
}

function _class_album_alter_parent(&$form) {
    //dump_msg($form['parent']);
    $form['parent']['#title'] = t('Parent album');
    $form['parent']['#multiple'] = 0;
    $form['parent']['#size'] = 0;
    $form['parent']['#required'] = 1;
    if (($arg = _path_match('node', 'add', 'acidfree', '%d'))) {
        $form['parent']['#default_value'] = $arg;
    }
    unset($form['parent']['#description']);
    if (is_array($form['parent']['#options'])) {
        foreach ($form['parent']['#options'] as $key => $value) {
            if ($value == '<'. t('root') .'>' && count($form['parent']['#default_value'])) {
                unset($form['parent']['#options'][$key]);
                continue;
            }
            if (is_array($value)) {
            }
            if (user_access('can upload to any album')) {
                continue;
            }
            $node = _album_from_tid($key);
            if ($node->share) {
                continue;
            }
            if (!node_access($node)) {
                unset($form['parent']['#options'][$key]);
                continue;
            }
        }
    }
}

function _class_album_form_alter($form_id, &$form) {
    $form['taxonomy'] = $form['parent'];
    unset($form['parent']);
}
?>
