<?php
/* $Id: class_video.inc,v 1.29 2006/11/09 00:15:29 vhmauery Exp $ */

/*
Acidfree Photo Albums for Drupal
Copyright (C) 2005 Vernon Mauery

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/

/**
 * @file
 * The is the class implementation of the photo.  All the acidfree elements
 * must be defined in this same manner, which makes creating a new class
 * relatively simple -- copy one of the existing classes and modify the 
 * relevant functions.
 */

require_once('image_manip.inc');

function class_video_register() {
    $class = new stdClass();
    if (!module_exists('video')) {
        return $class;
    }
    $class->class = 'video';
    $class->max_size = 4*(1 << 20);
    $class->mime_ext = Array(
            'video/x-msvideo'=>'avi',
            'video/mpeg'=>'mpg',
            'video/mp4'=>'mp4',
            'video/quicktime'=>'mov',
            'audio/x-pn-realaudio'=>'rm',
            'audio/x-realaudio'=>'rm',
            'application/vnd.rn-realmedia'=>'rm',
            'video/x-ms-wmv'=>'wmv',
            'video/x-ms-asf'=>'asf',
            'video/3gpp'=>'3gp',
        );
    $class->form_alter = '_class_video_form_alter';
    $class->nodeapi = '_class_video_nodeapi';
    $class->access = 'create videos';
    $class->add_form = '_class_video_add_form';
    $class->edit_form = '_class_video_edit_form';
    return $class;
}

function theme_acidfree_print_full_video(&$node) {
    if (!$node->large)
        return '';

    // get the pager
    $output = acidfree_pager_creator($node);

    $id = $node->nid;
    $name = $node->title;

    // icon link to the full size video
    $full_video = _acidfree_get_large_url($node, true);
    $output .= "<a href='$full_video'>".t('download video').'</a><br/>';

    $ext = pathinfo(_acidfree_get_large_path($node), PATHINFO_EXTENSION);
    switch ($ext) {
    case 'rm':
        $type = 'realmedia';
        break;
    case 'mp4':
    case 'mov':
        $type = 'quicktime';
        break;
    default:
        $type = 'windowsmedia';
        break;
    }
    $finfo = image_get_info(_acidfree_get_small_path($node, true));
    $node->videoy = $finfo['height'];
    $node->videox = $finfo['width'];
    $output .= acidfree_call("theme_video_{$type}_control",Array(&$node));
    $output .= "<div class='acidfree-body'>{$node->body}</div>\n";

    return $output;
}

function theme_acidfree_print_thumb_video(&$node, $parent=null) {
    if (count($node->parent) > 1 && $parent)
        $p = "pid={$parent->nid}";

    $image = l(theme('image', _acidfree_get_thumb_url($node),
                $node->title, $node->title, array('style'=>'border-left: none; border-right: none;', 'class'=>'acidfree'), false),
              "node/{$node->nid}", array(), $p, NULL, true, true);
    $size = variable_get('acidfree_thumb_dim', IMAGE_THUMB_SIZE) + 40;
    $cell .= "<div class='fullslide' style='width: {$size}px;'>\n";
    $size -= 20;
    $cell .= "<div class='outer video' style='width: {$size}px; height: {$size}px;'>\n";
    // XXX: this empty span is to make IE render vertical alignment correctly
    $cell .= "<span class='iefiller'></span>\n";
    $cell .= "$image";
    $cell .= "</div>\n";
    $cell .= "<div>{$node->title}</div>\n";
    $cell .= "</div>\n";
    return $cell;
}

function _class_video_add_form() {
  $node = (object)array('type' => 'video');
  return drupal_get_form('video_node_form', $node);
}

function _class_video_edit_form() {
}

function _class_video_form_alter($form_id, &$form) {
    //drupal_set_message("form_alter: $form_id (modified)");
    $form['taxonomy']['#weight'] = -19;
    if (($arg = _path_match('node', 'add', 'acidfree', 'video', '%d'))) {
        $acidfree_vid = acidfree_get_root();
        $form['taxonomy'][$acidfree_vid]['#default_value'] = $arg;
    }
    unset($form['log']);
    if (_path_match('album', '%d', 'contents')) {
        $form['video']['#collapsible'] = true;
        $form['video']['#collapsed'] = true;
    }
    $form['body_filter']['body']['#rows'] = 5;
    $attributes = array('onchange' => 'set_title(this.value);');
    $form['video_upload_file']['#attributes'] = 
        is_array($form['video_upload_file']['#attributes']) ? 
        array_merge($attributes, $form['image']['#attributes']):$attributes;
    if (($arg = _path_match('album', '%d', 'contents'))) {
        $form['body'] = $form['body_filter']['body'];
        unset($form['body_filter']);
        unset($form['video_upload_file']);
        unset($form['preview']);
        unset($form['image']);
    }

}
?>
